/*
--          This file is part of the New World OS and Objectify projects
--                  Copyright (C) 2008, 2009, 2010  QRW Software
--               J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
--
--   This program is free software: you can redistribute it and/or modify
--   it under the terms of the GNU General Public License as published by
--   the Free Software Foundation, either version 3 of the License, or
--   (at your option) any later version.
--
--   This program is distributed in the hope that it will be useful,
--   but WITHOUT ANY WARRANTY; without even the implied warranty of
--   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--   GNU General Public License for more details.
--
--   You should have received a copy of the GNU General Public License
--   along with this program, in the file LICENSE.  If not, see 
--   <http://www.gnu.org/licenses/>.
--
--   For the latest information, source code (SVN), releases, and bug tracking
--   go to:
--      http://savannah.nongnu.org/projects/objectify
--
--   For releases from Alpha_30 and up, bug and feature request tracking go to:
--      http://sourceforge.net/projects/objectify
--
--   For older bug tracking, releases and source code (CVS) prior to the
--   Alpha_30 release go to:
--      http://sourceforge.net/projects/nwos
--
--   Other related websites:
--      http://www.qrwsoftware.com
--      http://www.worldwide-database.org
--
--   You can also contact me via paper mail at:
--
--      QRW Software
--      P.O. Box 27511
--      Salt Lake City, UT 84127-0511, USA.
--
--   $Author: jsedwards $
--   $Date: 2010-01-17 15:01:51 -0700 (Sun, 17 Jan 2010) $
--   $Revision: 4463 $
--
--   NOTE: Subversion does not support the Log keyword so I have removed the
--   logs that were here when I was using CVS.  Use the "svn log" command to
--   see the revision history of this file.
--   (See http://subversion.tigris.org/faq.html#log-in-source)
--
*/


#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "objectify.h"
#include "strlcxx.h"


#define MAX_ENTRIES 64

typedef uint8 MD5[16];

typedef struct {
    char* name;
    MD5 md5;
} File_and_MD5;


static uint8 convert_char_to_hex_value(char c)
{
    uint8 result = 0xff;

    if (isxdigit(c))
    {
	if (isdigit(c))
	{
	    result = c - '0';
	}
	else if (isupper(c))
	{
	    result = c - 'A' + 10;
	}
	else
	{
	    result = c - 'a' + 10;
	}
    }

    return result;
}


int main(int argc, char* argv[])
{
    ObjRef ref;
    int i;
    int entry = 0;
    ObjCreateResult result;
    FILE* fp;
    File_and_MD5 entries[MAX_ENTRIES];
    char line[128];
    char *p;
    uint8 upper;
    uint8 lower;
    size_t len;

    if (argc != 2)
    {
	fprintf(stderr, "usage: %s md5_file\n", argv[0]);
	exit(1);
    }

    fp = fopen(argv[1], "r");

    if (fp == NULL)
    {
	perror(argv[1]);
	exit(1);
    }

    while (fgets(line, sizeof(line), fp) != NULL)
    {
	assert(entry < MAX_ENTRIES);

	p = line;

	for (i = 0; i < sizeof(MD5); i++)
	{
	    upper = convert_char_to_hex_value(*p++);

	    if (upper < 16)
	    {
		lower = convert_char_to_hex_value(*p++);
	    }

	    if (upper > 15 || lower > 15)
	    {
		break;
	    }

	    entries[entry].md5[i] = (upper << 4) | lower;
	}

	len = 0;

	if (i == sizeof(MD5) && *p++ == ' ')
	{
	    if (*p == ' ' || *p == '*')
	    {
		p++;                      /* point to the first character of the file name */
		*strchr(p, '\n') = '\0';  /* replace the newline with a null */
		len = strlen(p);
	    }
	}

	if (len == 0)
	{
	    fprintf(stderr, "invalid line: %s\n", line);
	    fclose(fp);
	    exit(1);
	}

	entries[entry].name = malloc(len + 1);

	if (entries[entry].name == NULL)
	{
	    perror("allocating memory for name");
	    fclose(fp);
	    exit(1);
	}

	strlcpy(entries[entry].name, p, len + 1);

	entry++;
    }

    if (ferror(fp))
    {
	perror(argv[1]);
	fclose(fp);
	exit(1);
    }

    fclose(fp);

    if (entry == 0)
    {
	fprintf(stderr, "Empty file: %s\n", argv[1]);
	exit(1);
    }


    nwos_log_arguments(argc, argv);

    nwos_initialize_objectify(PUBLIC, NULL);


    for (i = 0; i < entry; i++)
    {
	int j;

	for (j = 0; j < 16; j++)
	{
	    printf("%02x", entries[i].md5[j]);
	}
	printf("  %s\n", entries[i].name);

	result = nwos_create_file_with_only_md5(entries[i].name, entries[i].md5, &ref);

	assert(!is_void_reference(&ref));

	if (result == CREATED_NEW)
	{
	    printf("created new file path: %02x%02x%02x%02x\n", 
		   ref.id[0], ref.id[1], ref.id[2], ref.id[3]);
	}
	else
	{
	    printf("result: %d\n", result);
	}
    }


    nwos_terminate_objectify();

    return 0;
}


